/* Read PNM-file from stdin and write it to stdout */
/* For P6 with maxval > 255, create ASCII PPM output */

#include <stdio.h>

static int copy_in_to_out (void)

{int c;

 c = getc (stdin);
 while (c != EOF)
 {
   if (putc (c, stdout) == EOF) return 1;
   c = getc (stdin);
 }
 return 0;
}

typedef union {
  unsigned char uc[2];
  unsigned short us;
} MAP16;


int main (int argc, char *argv)

{char line[1024];
 char *ptr;
 char linecount;
 int width, height, maxval;
 int i1, i2;
 MAP16 m;

 if (argc > 1)
 {
   fprintf(stderr, "Usage: pnmbin < pnmfile\n");
   fprintf(stderr," Read PNM-file from stdin and write it to stdout.\n");
   fprintf(stderr," For binary PPM file with maxval > 255, write ASCII PPM\n");
   exit (0);
 }

 ptr = fgets (line, sizeof (line), stdin);
 if (ptr == NULL) return 0;

 if ((line[0] != 'P') || (line[1] != '6'))
 {
   fputs (line, stdout);
   return copy_in_to_out ();
 }
 /* Now we know that it is a PPM-file */

 /* Read width, height */
 for (;;)
 {
   ptr = fgets (line, sizeof (line), stdin);
   if (ptr == NULL)
   {
     fprintf (stderr, "Read error while expecting width/height\n");
     return 1;
   }
   if (line[0] != '#') break;
 }
 if (sscanf (line, "%d%d", &width, &height) != 2)
 {
   fprintf (stderr, "Could not read width/height\n");
   return 1;
 }

 /* Read maxval */
 for (;;)
 {
   ptr = fgets (line, sizeof (line), stdin);
   if (ptr == NULL)
   {
     fprintf (stderr, "Read error while expecting maxval\n");
     return 1;
   }
   if (line[0] != '#') break;
 }
 if (sscanf (line, "%d", &maxval) != 1)
 {
   fprintf (stderr, "Could not read maxval\n");
   return 1;
 }
 if (maxval <= 255)
 {
   fprintf (stdout, "P6\n%d %d\n%d\n", width, height, maxval);
   return copy_in_to_out ();
 }

 /* Create an ASCII PPM file */
 fprintf (stdout, "P3\n%d %d\n%d\n", width, height, maxval);
 for (;;)
 {
   i1 = getc (stdin);
   if (i1 == EOF) break;
   i2 = getc (stdin);
   if (i2 == EOF)
   {
     fprintf (stderr,"Image data has an odd number of bytes\n");
     break;
   }
   m.uc[0] = (unsigned char)i1;
   m.uc[1] = (unsigned char)i2;
   fprintf (stdout, "%d ", (int)m.us);
 }
 return 0;
}
